////////////////////////////////////////////////////////////////////////////////
// TITRE : DWEB - DATAWAREHOUSE ENGINEERING BENCHMARK - VERSION 0.4           //
//----------------------------------------------------------------------------//
// AUTEURS : JOUBERT Bruno / GUESMIA Sofiane                                  //
//           UNIVERSITE LYON 2 - MASTER IIDEE                                 //
//----------------------------------------------------------------------------//
// English translation by J. Darmont                                          //
// - Step 1: Interface (DONE)                                                 //
// - Step 2: Code/comments (TO DO)                                            //
//----------------------------------------------------------------------------//
// ENCADRES PAR : DARMONT Jrome                                              //
//                UNIVERSITE LYON 2 - LABORATOIRE ERIC                        //
//                http://eric.univ-lyon2.fr - - jerome.darmont@univ-lyon2.fr  //
//----------------------------------------------------------------------------//
// DESCRIPTION : Banc d'essai dcisionnel                                     //
//----------------------------------------------------------------------------//
// Cette application a pour but de permettre la cration d'un entrept de     //
// donnes synthtique et des charges associes.                              //
////////////////////////////////////////////////////////////////////////////////

package dweb03;

import java.sql.*;
import java.lang.*;

////////////////////////////////////////////////////////////////////////////////
// CLASSE DimensionH                                                          //
//----------------------------------------------------------------------------//
// Cette classe sert  modliser les dimensions de l'entrept.                //
////////////////////////////////////////////////////////////////////////////////

public class DimensionH {

////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTS                                                                  //
////////////////////////////////////////////////////////////////////////////////
  private String nomDimension;
  private int profondeurDimension;
  private int nbEnregistrementsHautNiveau;
  private int facteurEchelle;

  private int[] tNbAttributs;
  private String[] tNomTables;
  private TableDimensionH[] tTablesDimension;

////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTEURS                                                              //
////////////////////////////////////////////////////////////////////////////////

  public DimensionH(String pNom, int pProfondeur, int pEnrHtNiveau, int pFacteur) {
    this.nomDimension = pNom;
    this.profondeurDimension = pProfondeur;
    this.nbEnregistrementsHautNiveau = pEnrHtNiveau;
    this.facteurEchelle = pFacteur;
    this.tTablesDimension = new TableDimensionH[this.profondeurDimension];
    this.tNomTables = new String[this.profondeurDimension];
    this.tNbAttributs = new int[this.profondeurDimension];
  }

////////////////////////////////////////////////////////////////////////////////
// METHODES                                                                   //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNom - OK                                                       //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | nom de la dimension                                           //
////////////////////////////////////////////////////////////////////////////////
  public String getNom () {
    return this.nomDimension;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getInfos - OK                                                     //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | infos de la dimension en vue de l'affichage dans une boite de //
//     dialogue                                                               //
////////////////////////////////////////////////////////////////////////////////
  public String getInfos () {
    String infos = "";
    infos += this.nomDimension + " : Nb levels = " + Integer.toString(this.profondeurDimension)
                               + " / Nb of tuples in highest level = " + Integer.toString(this.nbEnregistrementsHautNiveau)
                               + " / Scale factor = " + Integer.toString(this.facteurEchelle);
    return infos;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getProfondeur - OK                                                //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | profondeur = nombre de tables dans la dimension                  //
////////////////////////////////////////////////////////////////////////////////
  public int getProfondeur () {
    return this.profondeurDimension;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNbEnregistrementsHautNiveau - OK                               //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | nombre d'enregistrements dans le plus haut niveau                //
////////////////////////////////////////////////////////////////////////////////
  public int getNbEnregistrementsHautNiveau () {
    return this.nbEnregistrementsHautNiveau;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getFacteurEchelle - OK                                            //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | facteur d'echelle au sein de la dimension                        //
////////////////////////////////////////////////////////////////////////////////
  public int getFacteurEchelle () {
    return this.facteurEchelle;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getTableI - OK                                                    //
//----------------------------------------------------------------------------//
// E : pNiveau | int | niveau de la table que l'on veut rcuprer             //
// S : TableDimensionH | table au niveau i si valide, null sinon              //
////////////////////////////////////////////////////////////////////////////////
  public TableDimensionH getTableI (int pNiveau) {
   if (pNiveau < 0 || pNiveau > this.profondeurDimension)
     return null;
   else
     return this.tTablesDimension[pNiveau];
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setNbAttributsTable - OK                                          //
//----------------------------------------------------------------------------//
// E : pNiveau | int | niveau de la table que l'on veut rcuprer             //
//     pNbAtt | int | nombre d'attributs de la table                          //
// S : boolean | resultat de la dfinition, test sur le param pNiveau         //
////////////////////////////////////////////////////////////////////////////////
  public boolean setNbAttributsTable( int pNiveau, int pNbAtt) {
    if (pNiveau < this.profondeurDimension && !(pNiveau < 0)) {
      this.tNbAttributs[pNiveau] = pNbAtt;
      return true;
    }
    else
      return false;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setNomTable - OK                                                  //
//----------------------------------------------------------------------------//
// E : pNiveau | int | niveau de la table que l'on veut rcuprer             //
//     pNomTable | String | nom de la table                                   //
// S : boolean | resultat de la dfinition, test sur le param pNiveau         //
////////////////////////////////////////////////////////////////////////////////
  public boolean setNomTable( int pNiveau, String pNomTable) {
    if (pNiveau < this.profondeurDimension && !(pNiveau < 0)) {
      this.tNomTables[pNiveau] = pNomTable;
      return true;
    }
    else
      return false;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setNomsTablesDefaut - OK                                          //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : \                                                                      //
//----------------------------------------------------------------------------//
// on dfinit des noms par dfaut pour les tables de la dimension             //
// --> "NOM_DIM_TABi"                                                         //
////////////////////////////////////////////////////////////////////////////////
  public void setNomsTablesDefaut () {
    int i;
    for (i=0; i<this.profondeurDimension; i++) {
      this.tNomTables[i] = this.nomDimension + "_TAB" + Integer.toString(i+1);
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION calculerTailleDimension - OK                                      //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : \                                                                      //
//----------------------------------------------------------------------------//
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
  public long calculerTailleDimension () {
    int i;
    long tailleDimension = 0;
    int nbEnr = this.nbEnregistrementsHautNiveau;

   // pour chaque niveau de la dimension
    for (i=this.profondeurDimension-1; i>=0; i--) {
      tailleDimension += (long)this.getTableI(i).calculerTailleEnr() * (long)nbEnr;
      // on calcule le nb d'enregistrements dans la table du niveau suivant
      nbEnr *= this.facteurEchelle;
    }
    return tailleDimension;
  }

////////////////////////////////////////////////////////////////////////////////
//FONCTION nbreEnregistrement - OK                                      //
//----------------------------------------------------------------------------//
//E : \                                                                      //
//S : \                                                                      //
//----------------------------------------------------------------------------//
////
////////////////////////////////////////////////////////////////////////////////
public long nbreEnregistrementDim (Connection pConnexion) {
	   long nbEnr = 0;
	   int i;
	   ResultSet rs;
	   String pRequete;

	   for(i=this.profondeurDimension-1; i>=0; i--){
		   pRequete = "SELECT COUNT(*) FROM "+(String)this.getTableI(i).getNom();

		   try{
			   Statement statement = pConnexion.createStatement();

			   rs = statement.executeQuery(pRequete);
			   rs.next();
			   nbEnr += rs.getInt(1);

			   statement.close();
		   }
		   catch (SQLException e) {
		   }
	   }

	   return nbEnr;
}

////////////////////////////////////////////////////////////////////////////////
// FONCTION genererDimension - OK                                             //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la dimension a bien t gnre                   //                                                                      //
////////////////////////////////////////////////////////////////////////////////
  public boolean genererDimension(Connection pConnexion, String[] pMessage) {
    int i;
    boolean resultat = true;

    // pour chaque niveau de la dimension
    for (i=this.profondeurDimension-1; i>=0; i--) {
      // on cr une table
      this.tTablesDimension[i] = new TableDimensionH(this.tNomTables[i], this.tNbAttributs[i]);

      // on cr le nom du champ de la cl primaire
      this.tTablesDimension[i].setClePrimaire("PK_" + this.tNomTables[i]);

      // dfinition des noms des champs de donnes et des types des champs de donnes de la table
      this.tTablesDimension[i].setNomsChampsDefaut();
      this.tTablesDimension[i].setTypesChampsDefaut();

      // on cr la table dans la base
      resultat &= this.tTablesDimension[i].creerTableBase(pConnexion, pMessage);

      // si ncessaire on cr galement une contrainte de cl etrangre pour rfrencer la table de niveau suprieur
      if (i < this.profondeurDimension-1)
        resultat &= this.tTablesDimension[i].setCleEtrangere(pConnexion,this.tTablesDimension[i+1],pMessage);
    }
    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION viderDimensionBase - OK                                           //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la dimension a bien t vide                     //                                                                      //
////////////////////////////////////////////////////////////////////////////////
  public boolean viderDimensionBase(Connection pConnexion, String[] pMessage) {
    int i;
    boolean resultat = true;

    for (i=0; i<this.profondeurDimension; i++) {
      resultat &= this.getTableI(i).viderTableBase(pConnexion, pMessage);
    }
    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION effacerDimensionBase - OK                                         //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la dimension a bien t efface de la base        //                                                                      //
////////////////////////////////////////////////////////////////////////////////
  public boolean effacerDimensionBase(Connection pConnexion, String[] pMessage) {
    int i;
    boolean resultat = true;

    for (i=0; i<this.profondeurDimension; i++) {
      resultat &= this.getTableI(i).effacerTableBase(pConnexion, pMessage);
      this.tTablesDimension[i] = null;
    }
    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
}
