////////////////////////////////////////////////////////////////////////////////
// TITRE : DWEB - DATAWAREHOUSE ENGINEERING BENCHMARK - VERSION 0.4           //
//----------------------------------------------------------------------------//
// AUTEURS : JOUBERT Bruno / GUESMIA Sofiane                                  //
//           UNIVERSITE LYON 2 - MASTER IIDEE                                 //
//----------------------------------------------------------------------------//
// English translation by J. Darmont                                          //
// - Step 1: Interface (DONE)                                                 //
// - Step 2: Code/comments (TO DO)                                            //
//----------------------------------------------------------------------------//
// ENCADRES PAR : DARMONT Jrome                                              //
//                UNIVERSITE LYON 2 - LABORATOIRE ERIC                        //
//                http://eric.univ-lyon2.fr - jerome.darmont@univ-lyon2.fr    //
//----------------------------------------------------------------------------//
// DESCRIPTION : Banc d'essai dcisionnel                                     //
//----------------------------------------------------------------------------//
// Cette application a pour but de permettre la cration d'un entrept de     //
// donnes synthtique et des charges associes.                              //
////////////////////////////////////////////////////////////////////////////////

package dweb03;

import java.sql.*;
import java.util.*;
import java.lang.*;
import java.io.*;

////////////////////////////////////////////////////////////////////////////////
// CLASSE EntrepotDonnees                                                     //
//----------------------------------------------------------------------------//
// Cette classe sert  modliser l'entrept de donnes.                       //
////////////////////////////////////////////////////////////////////////////////

public class EntrepotDonnees {

////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTS                                                                  //
////////////////////////////////////////////////////////////////////////////////
  private int nbTablesFaitTotal;
  private TableFait[] tTablesFait;

  private int nbDimensionsTotal;
  private DimensionH[] tDimensions;

////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTEURS                                                              //
////////////////////////////////////////////////////////////////////////////////

  public EntrepotDonnees(int pNbTFT, int pNbDimT) {
    this.nbTablesFaitTotal = pNbTFT;
    this.nbDimensionsTotal = pNbDimT;
    this.tTablesFait = new TableFait[this.nbTablesFaitTotal];
    this.tDimensions = new DimensionH[this.nbDimensionsTotal];
  }

////////////////////////////////////////////////////////////////////////////////
// METHODES                                                                   //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNbTablesFait - OK                                              //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | nb de tables de faits total dans l'entrept                      //
////////////////////////////////////////////////////////////////////////////////
  public int getNbTablesFait () {
    return this.nbTablesFaitTotal;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNbDimensions - OK                                              //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | nb de dimensions total dans l'entrept                           //
////////////////////////////////////////////////////////////////////////////////
  public int getNbDimensions () {
    return this.nbDimensionsTotal;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getInfos - OK                                                     //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | infos sur l'entrept en vue de l'affichage dans une           //
//     boite de dialogue                                                      //
////////////////////////////////////////////////////////////////////////////////
  public String getInfos () {
    int i = 0;
    double tailleMo;
    String infos = "";
    String sTailleMo = "";

    infos += "Data warehouse information:\n";
    infos += "\n";
    infos += "Total number of dimensions: " + Integer.toString(this.nbDimensionsTotal) + "\n";

    for (i=0; i<this.nbDimensionsTotal; i++) {
      infos += "  - " + this.getDimensionI(i).getInfos() + "\n";
    }

    infos += "\n";
    infos += "Total number of fact tables: " + Integer.toString(this.nbTablesFaitTotal) + "\n";

    for (i=0; i<this.nbTablesFaitTotal; i++) {
      infos += "  - " + this.getTableFaitI(i).getInfos() + "\n";
    }

    infos += "\n";
    tailleMo = (double)this.calculerTailleEntrepot() / (1024 * 1024);
    StringTokenizer st = new StringTokenizer (Double.toString(tailleMo),".");
    sTailleMo += st.nextToken() + "," + st.nextToken().substring(0,1);
    infos += "Warehouse size evaluation: " + sTailleMo + " MB";
    infos += "\n";

    return infos;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getCombinaisonDimensions - OK                                     //
//----------------------------------------------------------------------------//
// E : pTaille | int | taille de la combinaison  retourner                   //
// S : Vector | combinaison de "pTaille" dimensions prises parmi les          //
//     les dimensions de l'entrept --> pour association aux tables de faits  //
////////////////////////////////////////////////////////////////////////////////
  private Vector getCombinaisonDimensions (int pTaille) {
    int i;
    int j=0;
    Vector tableauDimEntrepot = new Vector();
    Vector tableauDimTF = new Vector(pTaille);

    for (i=0; i<this.getNbDimensions(); i++) {
      tableauDimEntrepot.addElement(this.getDimensionI(i));
    }
    while (j < pTaille) {
      Double D = new Double(Math.random() * this.getNbDimensions());
      if (!tableauDimTF.contains(tableauDimEntrepot.get(D.intValue()))) {
        tableauDimTF.addElement(tableauDimEntrepot.get(D.intValue()));
        j++;
      }
    }
    return tableauDimTF;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getDimensionI - OK                                                //
//----------------------------------------------------------------------------//
// E : pRang | int | rang de la dimension que l'on veut rcuprer             //
// S : DimensionH | dimension au rang i si valide, null sinon                 //
////////////////////////////////////////////////////////////////////////////////
  public DimensionH getDimensionI (int pRang) {
    if (pRang < 0 || pRang > this.nbDimensionsTotal)
      return null;
    else
      return this.tDimensions[pRang];
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getTableFaitI - OK                                                //
//----------------------------------------------------------------------------//
// E : pRang | int | rang de la table de faits que l'on veut rcuprer        //
// S : TableFait | table de faits au rang i si valide, null sinon             //
////////////////////////////////////////////////////////////////////////////////
  public TableFait getTableFaitI (int pRang) {
    if (pRang < 0 || pRang > this.nbTablesFaitTotal)
      return null;
    else
      return this.tTablesFait[pRang];
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION ajouterDimension - OK                                             //
//----------------------------------------------------------------------------//
// E : pRang | int | rang d'insertion de la dimension dans le tableau         //
//     pDimension | DimensionH | dimension  ajouter  l'entrept             //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la dimension a bien t ajoute  l'entrept      //
////////////////////////////////////////////////////////////////////////////////
  public boolean ajouterDimension (int pRang, DimensionH pDimension, String[] pMessage) {
    if (pRang < 0 || pRang > this.nbDimensionsTotal) {
      pMessage[0] += "Error when adding dimension " + pDimension.getNom() + " into the warehouse\n";
      return false;
    }
    else {
      this.tDimensions[pRang] = pDimension;
      pMessage[0] += "Dimension " + pDimension.getNom() + " added into the warehouse\n";
      return true;
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION ajouterTableFait - OK                                             //
//----------------------------------------------------------------------------//
// E : pRang | int | rang d'insertion de la table de faits dans le tableau    //
//     pTableFait | TableFait | table de faits  ajouter  l'entrept         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la table de faits a bien t ajoute  l'entrept //
////////////////////////////////////////////////////////////////////////////////
  public boolean ajouterTableFait (int pRang, TableFait pTableFait, String[] pMessage) {
    if (pRang < 0 || pRang > this.nbTablesFaitTotal) {
      pMessage[0] += "Error when adding fact table " + pTableFait.getNom() + " into the warehouse\n";
      return false;
    }
    else {
      this.tTablesFait[pRang] = pTableFait;
      pMessage[0] += "Fact table " + pTableFait.getNom() + " added into the warehouse\n";
      return true;
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION genererEntrepotHN - OK                                            //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection                                                //
//     pMNbDimsTF | int | nb moyen de dimensions par tables de faits          //
//     pNbMesures | int | nb moyen de mesures dans les tables de faits        //
//     pDensite | double | moyenne des densits des tables de faits           //
//     pMNbNiveaux | int | nb moyen de niveaux dans les dimensions            //
//     pMNbAttributs | int | nb moyens d'attributs dans les niveaux           //
//     pMNbEnrHN | int | nb moyen d'enregistrements dans les hauts niveaux    //
//     pMFacteurEchelle | int | facteur d'chelle moyen entre les niveaux     //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si l'entrept  bien t cr                        //
//----------------------------------------------------------------------------//
// Parametrage Haut Niveau : l'utilisateur ne saisit que les valeurs moyennes //
// On doit donc dterminer les valeurs relles  partir de ces valeurs        //
// --> Vr = NbAl * (Vm/2)^(1/2) + Vm                                          //
//     avec NbAl qui suit une loi normale N(0,1)                              //
////////////////////////////////////////////////////////////////////////////////
  public boolean genererEntrepotHN(Connection pConnexion, int pMNbDimsTF, int pMNbMesures, double pMDensite, int pMNbNiveaux, int pMNbAttributs, int pMNbEnrHN, int pMFacteurEchelle, String[] pMessage) {
    int i, j;
    int nbDimsTF, nbMesures, nbNiveaux, nbAttributs, nbEnrHN, facteurEchelle;
    double densite;
    boolean resultat = true;
    Random generateur = new Random( );

    // cration et gnration des dimensions de l'entrept
    for (i=0; i<this.getNbDimensions(); i++) {
      // on tire les valeurs au hasard
      nbNiveaux = (int)(generateur.nextGaussian() * Math.sqrt(pMNbNiveaux/2) + pMNbNiveaux);
      if (nbNiveaux == 0)
        nbNiveaux = 1;

      nbEnrHN = (int)(generateur.nextGaussian() * Math.sqrt(pMNbEnrHN/2) + pMNbEnrHN);
      if (nbEnrHN == 0)
        nbEnrHN = 1;

      facteurEchelle = (int)(generateur.nextGaussian() * Math.sqrt(pMFacteurEchelle/2) + pMFacteurEchelle);
      if (facteurEchelle == 0)
        facteurEchelle = 1;

      // cration de la dimension et ajout a l'entrept
      this.ajouterDimension(i, new DimensionH("DIM"+Integer.toString(i+1), nbNiveaux, nbEnrHN, facteurEchelle) , pMessage);
      this.getDimensionI(i).setNomsTablesDefaut();

      // pour chaque table de la dimension on dtermine le nombre d'attributs
      for (j=0; j<this.getDimensionI(i).getProfondeur(); j++) {
        nbAttributs = (int)Math.ceil(generateur.nextGaussian() * Math.sqrt(pMNbAttributs/2) + pMNbAttributs);
        if (nbAttributs == 0)
          nbAttributs = 1;
        this.getDimensionI(i).setNbAttributsTable(j, nbAttributs);
      }
      // gnration de la dimension dans la base de donnes
      resultat &= this.getDimensionI(i).genererDimension(pConnexion,pMessage);
    }

    // cration et gnration des tables de faits de l'entrept
    for (i=0; i<this.getNbTablesFait(); i++) {
      Vector dimensionsTF = new Vector();

      // on tire les valeurs au hasard
      nbDimsTF = (int)Math.ceil(generateur.nextGaussian() * Math.sqrt(pMNbDimsTF/2) + pMNbDimsTF);
      if (nbDimsTF == 0)
          nbDimsTF = 1;

      nbMesures = (int)Math.ceil(generateur.nextGaussian() * Math.sqrt(pMNbMesures/2) + pMNbMesures);
      if (nbMesures == 0)
          nbMesures = 1;

      if (pMDensite < 0.5)
        densite = (generateur.nextGaussian() * pMDensite/2) + pMDensite;
      else
        densite = (generateur.nextGaussian() * (1-pMDensite)/2) + pMDensite;

      // cration de la table de faits et ajout a l'entrept
      this.ajouterTableFait(i, new TableFait("FACT_TABLE" + Integer.toString(i+1), densite, nbMesures, nbDimsTF), pMessage);
      this.getTableFaitI(i).setNomsChampsMesuresDefaut();

      // si le nb de dimensions de la table de faits est suprieur au nb de dimensions de l'entrept : corection
      if (this.getTableFaitI(i).getNbDimensions() > this.getNbDimensions())
        this.getTableFaitI(i).setNbDimensions(this.getNbDimensions());

      // on determine la combinaison de dimensions qui vont tre associes  la table de faits
      dimensionsTF = getCombinaisonDimensions (this.getTableFaitI(i).getNbDimensions());

      // on ajoute les dimensions de la combinaison  la table de faits
      for (j=0; j<this.getTableFaitI(i).getNbDimensions(); j++) {
        this.getTableFaitI(i).ajouterDimension(j,(DimensionH)dimensionsTF.get(j),pMessage);
      }

      // gnration de la table de faits dans la base de donnes
      resultat &= this.getTableFaitI(i).genererTableFait(pConnexion,pMessage);
    }
    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION genererEntrepotBN - OK                                            //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection                                                //
//     pDimensions | ModeleTableDimension | tableau des parametres complets   //
//                                          des dimensions                    //
//     pTablesFaits | ModeleTableTF | tableau des parametres complets         //
//                                    des tables de faits                     //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si l'entrept  bien t cr                        //
////////////////////////////////////////////////////////////////////////////////
  public boolean genererEntrepotBN (Connection pConnexion, ModeleTableDimension pDimensions, ModeleTableTF pTablesFaits, String[] pMessage) {
    boolean resultat = true;
    int i, j, temp;

    // cration et gnration des dimensions de l'entrept
    for (i=0; i<pDimensions.getRowCount(); i++) {
      // cration de la dimension et ajout a l'entrept
       this.ajouterDimension(i,
                            new DimensionH("DIM" + Integer.toString(i+1),
                                           Integer.parseInt((String)pDimensions.getValueAt(i,1)),
                                           Integer.parseInt((String)pDimensions.getValueAt(i,2)),
                                           Integer.parseInt((String)pDimensions.getValueAt(i,3))),
                            pMessage);

      this.getDimensionI(i).setNomsTablesDefaut();

      // pour chaque table de la dimension on fixe le nombre d'attributs
      StringTokenizer st = new StringTokenizer ((String)pDimensions.getValueAt(i,4),"/");
      j=0;
      while (st.hasMoreTokens ()) {
        temp = Integer.parseInt(st.nextToken());
        this.getDimensionI(i).setNbAttributsTable(j,temp);
        j++;
      }

      // gnration de la dimension dans la base de donnes
      resultat &= this.getDimensionI(i).genererDimension(pConnexion, pMessage);
    }

    // cration et gnration des tables de faits de l'entrept
    for (i=0; i<pTablesFaits.getRowCount(); i++) {
      Vector dimensionsTF = new Vector();

      // cration de la table de faits et ajout a l'entrept
      this.ajouterTableFait(i,
                            new TableFait("FACT_TABLE" + Integer.toString(i+1),
                                           Double.parseDouble((String)pTablesFaits.getValueAt(i,3)),
                                           Integer.parseInt((String)pTablesFaits.getValueAt(i,2)),
                                           Integer.parseInt((String)pTablesFaits.getValueAt(i,1))),
                            pMessage);
      this.getTableFaitI(i).setNomsChampsMesuresDefaut();

      // si le nb de dimensions de la table de faits est suprieur au nb de dimensions de l'entrept : corection
      if (this.getTableFaitI(i).getNbDimensions() > this.getNbDimensions())
        this.getTableFaitI(i).setNbDimensions(this.getNbDimensions());

      // on determine la combinaison de dimensions qui vont tre associes  la table de faits
      dimensionsTF = getCombinaisonDimensions (this.getTableFaitI(i).getNbDimensions());

      // on ajoute les dimensions de la combinaison  la table de faits
      for (j=0; j<this.getTableFaitI(i).getNbDimensions(); j++) {
        this.getTableFaitI(i).ajouterDimension(j,(DimensionH)dimensionsTF.get(j),pMessage);
      }

      // gnration de la table de faits dans la base de donnes
      resultat &= this.getTableFaitI(i).genererTableFait(pConnexion,pMessage);
    }
    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION viderEntrepot - OK                                                //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection                                                //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si l'entrept  bien t effac de la base           //
////////////////////////////////////////////////////////////////////////////////
  public boolean viderEntrepot (Connection pConnexion, String[] pMessage) {
    boolean resultat = true;
    int i;

    for (i=0; i<this.nbTablesFaitTotal; i++) {
      resultat &= this.getTableFaitI(i).viderTableBase(pConnexion, pMessage);
    }
    for (i=0; i<this.nbDimensionsTotal; i++) {
      resultat &= this.getDimensionI(i).viderDimensionBase(pConnexion, pMessage);
    }
    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION effacerEntrepotBase - OK                                          //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection                                                //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si l'entrept  bien t effac de la base           //
////////////////////////////////////////////////////////////////////////////////
  public boolean effacerEntrepotBase (Connection pConnexion, String[] pMessage) {
    boolean resultat = true;
    int i;

    for (i=0; i<this.nbTablesFaitTotal; i++) {
      resultat &= this.getTableFaitI(i).effacerTableBase(pConnexion, pMessage);
      this.tTablesFait[i] = null;
    }
    for (i=0; i<this.nbDimensionsTotal; i++) {
      resultat &= this.getDimensionI(i).effacerDimensionBase(pConnexion, pMessage);
      this.tDimensions[i] = null;
    }
    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION calculerTailleEntrepot - OK                                       //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | taille de l'entrepot                                             //
////////////////////////////////////////////////////////////////////////////////
  public long calculerTailleEntrepot () {
    int i;
    long tailleEntrepot = 0;

    for (i=0; i<this.nbDimensionsTotal; i++) {
      tailleEntrepot += this.getDimensionI(i).calculerTailleDimension();
    }

    for (i=0; i<this.nbTablesFaitTotal; i++) {
      tailleEntrepot += this.getTableFaitI(i).calculerTaille();
    }
    return tailleEntrepot;
  }

////////////////////////////////////////////////////////////////////////////////
//FONCTION nbreEnregistrement - OK                                       //
//----------------------------------------------------------------------------//
//E : \                                                                      //
//S : int | taille de l'entrepot                                             //
////////////////////////////////////////////////////////////////////////////////
 public int nbreEnregistrement (Connection pConnexion) {
   int i;
   int nbreEnr = 0;

   for (i=0; i<this.nbDimensionsTotal; i++) {
	   nbreEnr += this.getDimensionI(i).nbreEnregistrementDim(pConnexion);
   }

   for (i=0; i<this.nbTablesFaitTotal; i++) {
	   nbreEnr += this.getTableFaitI(i).nbreEnregistrementFait(pConnexion);
   }
   return nbreEnr;
 }

////////////////////////////////////////////////////////////////////////////////
// FONCTION genererRequetes - OK                                              //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : boolean | rsultat de la gnration                                    //
////////////////////////////////////////////////////////////////////////////////
  public boolean genererRequetes (String nomFichier, int pNbRequetes, int pNbMAttributs, int pNbMRestrictions, int pNbMAgregats, double pProbOLAP, double pProbCUBE, double pProbHAVING, int pNbMDrillDowns) {
    int i,j,k;

    int nbAttributs, nbRestrictions, nbAgregats, nbDrillDowns, profondeurDD;
    int profondeurAl, noAttributAl, noMesureAl, noAgregatAl;

    boolean isROLAP;
    String lAttributs, lTables, lConditions, lAttAgregats, lAgregats, lGroupBy, opGroupBy, clGroupBy, clHaving;

    TableFait tableFaitsAl;
    DimensionH dimensionAl, dimensionDD;

    Random generateur = new Random( );
    PrintWriter monFichier;

    try {
      monFichier = new PrintWriter(new BufferedWriter(new FileWriter(nomFichier)));

      i = 0;
      while (i < pNbRequetes) {
        // On suppose la requete simple
        isROLAP = false;

        // On initialise les chaines qui vont composer la requte
        lAttributs = "";
        lTables = "";
        lConditions = "";
        lAttAgregats = "";
        lAgregats = "";
        lGroupBy = "";
        opGroupBy = "";
        clGroupBy = "";
        clHaving = "";

        // On choisit une table de faits au hasard
        tableFaitsAl = this.getTableFaitI(generateur.nextInt(this.nbTablesFaitTotal));
        lTables += tableFaitsAl.getNom();

        nbAttributs = (int)(pNbMAttributs + generateur.nextGaussian() * Math.sqrt(pNbMAttributs/2));
        if (nbAttributs == 0)
          nbAttributs = 1;

        nbRestrictions = (int)(pNbMRestrictions + generateur.nextGaussian() * Math.sqrt(pNbMRestrictions/2));
        if (nbRestrictions == 0)
          nbRestrictions = 1;

        profondeurAl = 0;
        dimensionAl = null;

        // boucle sur le nb d'attributs slectionns
        for (j=0; j<nbAttributs; j++) {
          // On choisit une dimension au hasard
          dimensionAl = tableFaitsAl.getDimensionI(generateur.nextInt(tableFaitsAl.getNbDimensions()));

          // On se place  une profondeur donne dans la dimension dimensionAl
          profondeurAl = generateur.nextInt(dimensionAl.getProfondeur());

          for (k=0; k<=profondeurAl; k++) {
            // On n'ajoute le nom de la table que s'il ne figure pas deja dans la liste des tables + ajout de la jointure avec la table prcdente
            if (lTables.indexOf(dimensionAl.getTableI(k).getNom()) == -1) {
              lTables += ", " + dimensionAl.getTableI(k).getNom();
              if (k == 0)
                // jointure entre la table de faits et les dimensions
                lConditions += ((j==0) ? "" : " AND ") + "FK_" + dimensionAl.getTableI(k).getClePrimaire() + " = " + dimensionAl.getTableI(k).getClePrimaire();
              else
                // jointure entre les niveaux des dimensions
                lConditions += " AND " + dimensionAl.getTableI(k-1).getCleEtrangere() + " = " + dimensionAl.getTableI(k).getClePrimaire();
            }
          }

          // On selectionne un champ au hasard dans la table a la profondeur dimensionAl
          noAttributAl = generateur.nextInt(dimensionAl.getTableI(profondeurAl).getNbChamps());
          // On n'ajoute le champ que s'il ne figure pas deja dans la liste des champs
          if (lAttributs.indexOf(dimensionAl.getTableI(profondeurAl).getNomChamp(noAttributAl)) == -1){
            lAttributs += ((j==0) ? "" : ", ") + dimensionAl.getTableI(profondeurAl).getNomChamp(noAttributAl);
            // on determine s'il y a une restriction et si oui on complete la liste des conditions
            if (generateur.nextDouble() < ((double)nbRestrictions / (double)nbAttributs))
              lConditions += " AND " + dimensionAl.getTableI(profondeurAl).getNomChamp(noAttributAl) + " > DBMS_RANDOM.string('a',25)";
          }
        }

        // on enregistre les valeurs si requetes DRILL DOWN
        profondeurDD = profondeurAl - 1;
        dimensionDD = dimensionAl;

        // Si la requete est une requete OLAP
        if (generateur.nextDouble() < pProbOLAP) {
          isROLAP = true;
          // Agregat
          nbAgregats = (int)(pNbMAgregats + generateur.nextGaussian() * Math.sqrt(pNbMAgregats/2));
          if (nbAgregats == 0)
            nbAgregats = 1;

          for (j=0; j<nbAgregats; j++) {
            noMesureAl = generateur.nextInt(tableFaitsAl.getNbMesures());
            if (lAgregats.indexOf(tableFaitsAl.getNomChampMesure(noMesureAl)) == -1) {
              lAttAgregats += ((j==0) ? "" : ", ") + tableFaitsAl.getNomChampMesure(noMesureAl);
              lAgregats += ", " + "SUM(" + tableFaitsAl.getNomChampMesure(noMesureAl) + ")";
            }
          }

          // Clause Group By
          lGroupBy = lAttributs;

          if (generateur.nextDouble() < pProbCUBE)
            opGroupBy = "CUBE";
          else
            opGroupBy = "ROLLUP";

          clGroupBy = "GROUP BY " + opGroupBy + " (" + lGroupBy + ")";

          // Clause Having
          if (generateur.nextDouble() < pProbHAVING) {
            StringTokenizer st = new StringTokenizer (lAgregats,",");

            noAgregatAl = generateur.nextInt(st.countTokens());
            for (j=0; j<noAgregatAl-1; j++) {
              st.nextToken();
            }
            clHaving = "HAVING " + st.nextToken() + " >= " + Integer.toString(generateur.nextInt());
          }
        }

        if (isROLAP)
          monFichier.println("-- OLAP query");
        else
          monFichier.println("-- Extraction query");

        monFichier.println("SELECT " + lAttributs + " " + lAgregats + " FROM " + lTables + " WHERE " + lConditions + " " + clGroupBy + " " + clHaving);
        i++;

        // gnration des requtes DRILL DOWN
        if (isROLAP) {
          nbDrillDowns = (int)(pNbMDrillDowns + generateur.nextGaussian() * Math.sqrt(pNbMDrillDowns/2));
          while (nbDrillDowns > 0 && profondeurDD >= 0) {
            noAttributAl = generateur.nextInt(dimensionDD.getTableI(profondeurDD).getNbChamps());
            lAttributs += ", " + dimensionDD.getTableI(profondeurDD).getNomChamp(noAttributAl);
            lGroupBy += ", " + dimensionDD.getTableI(profondeurDD).getNomChamp(noAttributAl);
            clGroupBy = "GROUP BY " + opGroupBy + " (" + lGroupBy + ")";

            monFichier.println("-- Drill-down query");
            monFichier.println("SELECT " + lAttributs + " " + lAgregats + " FROM " + lTables + " WHERE " + lConditions + " " + clGroupBy + " " + clHaving);

            profondeurDD--;
            nbDrillDowns--;
            i++;
          }
        }
      }
      monFichier.close();
      return true;
    }
    catch (IOException ex) {
      return false;
    }
  }

////////////////////////////////////////////////////////////////////////////////
}
