////////////////////////////////////////////////////////////////////////////////
// TITRE : DWEB - DATAWAREHOUSE ENGINEERING BENCHMARK - VERSION 0.4           //
//----------------------------------------------------------------------------//
// AUTEURS : JOUBERT Bruno / GUESMIA Sofiane                                  //
//           UNIVERSITE LYON 2 - MASTER IIDEE                                 //
//----------------------------------------------------------------------------//
// English translation by J. Darmont                                          //
// - Step 1: Interface (DONE)                                                 //
// - Step 2: Code/comments (TO DO)                                            //
//----------------------------------------------------------------------------//
// ENCADRES PAR : DARMONT Jrome                                              //
//                UNIVERSITE LYON 2 - LABORATOIRE ERIC                        //
//                http://eric.univ-lyon2.fr - - jerome.darmont@univ-lyon2.fr  //
//----------------------------------------------------------------------------//
// DESCRIPTION : Banc d'essai dcisionnel                                     //
//----------------------------------------------------------------------------//
// Cette application a pour but de permettre la cration d'un entrept de     //
// donnes synthtique et des charges associes.                              //
////////////////////////////////////////////////////////////////////////////////

package dweb03;

import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import java.lang.*;
import java.lang.Number.*;

////////////////////////////////////////////////////////////////////////////////
// CLASSE ModeleTableDimension                                                //
//----------------------------------------------------------------------------//
// Cette classe est utilise pour la saise des paramtres avancs (bas niveau)//
// dans la fentre DLGParametresBN (JTable de saisie des dimensions)          //
// Les vecteurs dfinissent les noms des colonnes et les donnes du tableau   //
// Une instance de cette classe est transmise  la fonction genererEntrepotBN //
// de la classe "EntreptDonnes".                                            //
////////////////////////////////////////////////////////////////////////////////

public class ModeleTableDimension extends AbstractTableModel {

////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTS                                                                  //
////////////////////////////////////////////////////////////////////////////////
  protected Vector donnes;
  protected Vector nomsColonnes;
  protected int nbDimensions;

////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTEURS                                                              //
////////////////////////////////////////////////////////////////////////////////

  public ModeleTableDimension () {
    this.nomsColonnes = new Vector();
    this.nomsColonnes.add("DIMENSION");
    this.nomsColonnes.add("NB_LEVELS");
    this.nomsColonnes.add("HHLEVEL_SIZE");
    this.nomsColonnes.add("DIM_SFACTOR");
    this.nomsColonnes.add("NB_ATT");
    this.donnes = new Vector();
  }

////////////////////////////////////////////////////////////////////////////////
// METHODES                                                                   //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Quand le nombre de dimensions change on vide la table puis on la remplit  //
// nouveau avec le bon nombre de lignes                                       //
////////////////////////////////////////////////////////////////////////////////
  public void ajouterDimensions(int pNbDimensions) {
    int i;
    this.nbDimensions = pNbDimensions;
    this.donnes.clear();
    for (i=0; i<this.nbDimensions; i++) {
      this.donnes.add("Dimension #" + Integer.toString(i+1));
      this.donnes.add("3");
      this.donnes.add("10");
      this.donnes.add("10");
      this.donnes.add("5/5/5");
    }
  }

////////////////////////////////////////////////////////////////////////////////
  public int getColumnCount() {
    return nomsColonnes.size();
  }

////////////////////////////////////////////////////////////////////////////////
  public int getRowCount() {
    return donnes.size()/getColumnCount();
  }

////////////////////////////////////////////////////////////////////////////////
  public String getColumnName(int columnIndex) {
    String colName="";
    if (columnIndex<=getColumnCount())
      colName=(String)nomsColonnes.elementAt(columnIndex);
    return colName;
  }

////////////////////////////////////////////////////////////////////////////////
  public Class getColumnClass(int columnIndex){
    return String.class;
  }

////////////////////////////////////////////////////////////////////////////////
// Dfinition des cellules ditables dans la JTable                           //
////////////////////////////////////////////////////////////////////////////////
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    if (columnIndex == 0)
      return false;
    else
      return true;
  }

////////////////////////////////////////////////////////////////////////////////
  public Object getValueAt(int rowIndex, int columnIndex) {
    return (String)donnes.elementAt((rowIndex*getColumnCount())+columnIndex);
  }

////////////////////////////////////////////////////////////////////////////////
// Fonction de vrification des valeurs saisies par l'utilisateur avant       //
// validation du changement dans le vecteur "Donnes"                         //
////////////////////////////////////////////////////////////////////////////////
  public void setValueAt(Object valeur, int rowIndex, int columnIndex) {
    try {
      // Si on change le nombre de tables de la dimension --> chgt de la liste du nombre des attributs des tables
      if (columnIndex == 1) {
        Integer nbTables = new Integer(valeur.toString());
        if (nbTables.intValue()>0){
          String champs = "5";
          for (int i=1; i<nbTables.intValue(); i++) {
            champs += "/5";
          }
          this.donnes.setElementAt(valeur.toString(), rowIndex * getColumnCount() + columnIndex);
          this.setValueAt(champs, rowIndex , 4);
          this.fireTableDataChanged();
        }
        else
          JOptionPane.showMessageDialog(null,"Please enter positive values");
      }
      // Si on change la liste du nombre des attributs --> vrif du champ saisi
      else if (columnIndex == 4) {
        StringTokenizer st = new StringTokenizer(valeur.toString(),"/");
        if (st.countTokens() != Integer.parseInt(this.getValueAt(rowIndex,1).toString())){
          JOptionPane.showMessageDialog(null,"Please enter " + this.getValueAt(rowIndex,1).toString() + " values separated by \"/\"");
        }
        else {
          boolean chaineOK = true;
          while (st.hasMoreTokens()) {
            chaineOK &= Integer.parseInt(st.nextToken()) > 0;
          }
          if (chaineOK) {
            this.donnes.setElementAt(valeur.toString(), rowIndex * getColumnCount() + columnIndex);
            this.fireTableDataChanged();
          }
          else
            JOptionPane.showMessageDialog(null,"Please enter positive values");
        }
      }
      // Sinon on vrifie que la valeur saisie est un entier
      else {
        if (Integer.parseInt(valeur.toString()) > 0) {
          this.donnes.setElementAt(valeur.toString(), rowIndex * getColumnCount() + columnIndex);
          this.fireTableDataChanged();
        }
        else
          JOptionPane.showMessageDialog(null,"Please enter positive values");
      }
    }
    catch (NumberFormatException e) {
      JOptionPane.showMessageDialog(null,"Please enter integer values");
    }
  }
////////////////////////////////////////////////////////////////////////////////
}

