////////////////////////////////////////////////////////////////////////////////
// TITRE : DWEB - DATAWAREHOUSE ENGINEERING BENCHMARK - VERSION 0.4           //
//----------------------------------------------------------------------------//
// AUTEURS : JOUBERT Bruno / GUESMIA Sofiane                                  //
//           UNIVERSITE LYON 2 - MASTER IIDEE                                 //
//----------------------------------------------------------------------------//
// English translation by J. Darmont                                          //
// - Step 1: Interface (DONE)                                                 //
// - Step 2: Code/comments (TO DO)                                            //
//----------------------------------------------------------------------------//
// ENCADRES PAR : DARMONT Jrome                                              //
//                UNIVERSITE LYON 2 - LABORATOIRE ERIC                        //
//                http://eric.univ-lyon2.fr - - jerome.darmont@univ-lyon2.fr  //
//----------------------------------------------------------------------------//
// DESCRIPTION : Banc d'essai dcisionnel                                     //
//----------------------------------------------------------------------------//
// Cette application a pour but de permettre la cration d'un entrept de     //
// donnes synthtique et des charges associes.                              //
////////////////////////////////////////////////////////////////////////////////

package dweb03;

import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

////////////////////////////////////////////////////////////////////////////////
// CLASSE ModeleTableTF                                                       //
//----------------------------------------------------------------------------//
// Cette classe est utilise pour la saise des paramtres avancs (bas niveau)//
// dans la fentre DLGParametresBN (JTable de saisie des tables de fait)      //
// Les vecteurs dfinissent les noms des colonnes et les donnes du tableau   //
// Une instance de cette classe est transmise  la fonction genererEntrepotBN //
// de la classe "EntreptDonnes".                                            //
////////////////////////////////////////////////////////////////////////////////

public class ModeleTableTF extends AbstractTableModel {

////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTS                                                                  //
////////////////////////////////////////////////////////////////////////////////
  protected Vector donnes;
  protected Vector nomsColonnes;
  protected int nbTablesFaits;

////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTEURS                                                              //
////////////////////////////////////////////////////////////////////////////////

  public ModeleTableTF () {
    this.nomsColonnes = new Vector();
    this.nomsColonnes.add("FACT TABLE");
    this.nomsColonnes.add("NB_DIM");
    this.nomsColonnes.add("NB_MEAS");
    this.nomsColonnes.add("DENSITY");
    this.donnes = new Vector();
  }

////////////////////////////////////////////////////////////////////////////////
// METHODES                                                                   //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Quand le nombre de tables de faits change on vide la table puis on la      //
// remplit  nouveau avec le bon nombre de lignes                             //
////////////////////////////////////////////////////////////////////////////////
  public void ajouterTablesFaits(int pNbTables) {
    int i;
    this.donnes.clear();
    for (i=0; i<pNbTables; i++) {
      this.donnes.add("Fact table #" + Integer.toString(i+1));
      this.donnes.add("5");
      this.donnes.add("5");
      this.donnes.add("0.6");
    }
  }

////////////////////////////////////////////////////////////////////////////////
  public int getColumnCount() {
    return nomsColonnes.size();
  }

////////////////////////////////////////////////////////////////////////////////
  public int getRowCount() {
    return donnes.size()/getColumnCount();
  }

////////////////////////////////////////////////////////////////////////////////
  public String getColumnName(int columnIndex) {
    String colName="";
    if (columnIndex<=getColumnCount())
      colName=(String)nomsColonnes.elementAt(columnIndex);
    return colName;
  }

////////////////////////////////////////////////////////////////////////////////
  public Class getColumnClass(int columnIndex){
    return String.class;
  }

////////////////////////////////////////////////////////////////////////////////
// Dfinition des cellules ditables dans la JTable                           //
////////////////////////////////////////////////////////////////////////////////
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    if (columnIndex == 0)
      return false;
    else
      return true;
  }

////////////////////////////////////////////////////////////////////////////////
  public Object getValueAt(int rowIndex, int columnIndex) {
    return (String)donnes.elementAt((rowIndex*getColumnCount())+columnIndex);
  }

////////////////////////////////////////////////////////////////////////////////
// Fonction de vrification des valeurs saisies par l'utilisateur avant       //
// validation du changement dans le vecteur "Donnes"                         //
////////////////////////////////////////////////////////////////////////////////
  public void setValueAt(Object valeur, int rowIndex, int columnIndex) {
    if (columnIndex == 3){
      try {
        Double dValeur = new Double(valeur.toString());
        if (dValeur.doubleValue() < 0.0 || dValeur.doubleValue() > 1.0)
          JOptionPane.showMessageDialog(null,"Please enter a value between 0.0 and 1.0");
        else {
          this.donnes.setElementAt(valeur.toString(), rowIndex * getColumnCount() + columnIndex);
          this.fireTableCellUpdated(rowIndex,columnIndex);
        }
      }
      catch (NumberFormatException e) {
        JOptionPane.showMessageDialog(null,"Please enter a value between 0.0 and 1.0");
      }
    }
    else {
      try {
        if (Integer.parseInt(valeur.toString()) > 0) {
          this.donnes.setElementAt(valeur.toString(), rowIndex * getColumnCount() + columnIndex);
          this.fireTableCellUpdated(rowIndex, columnIndex);
        }
        else
          JOptionPane.showMessageDialog(null,"Please enter positive values");
      }
      catch (NumberFormatException e) {
        JOptionPane.showMessageDialog(null,"Please enter an integer");
      }
    }
  }
////////////////////////////////////////////////////////////////////////////////
}
