////////////////////////////////////////////////////////////////////////////////
// TITRE : DWEB - DATAWAREHOUSE ENGINEERING BENCHMARK - VERSION 0.4           //
//----------------------------------------------------------------------------//
// AUTEURS : JOUBERT Bruno / GUESMIA Sofiane                                  //
//           UNIVERSITE LYON 2 - MASTER IIDEE                                 //
//----------------------------------------------------------------------------//
// English translation by J. Darmont                                          //
// - Step 1: Interface (DONE)                                                 //
// - Step 2: Code/comments (TO DO)                                            //
//----------------------------------------------------------------------------//
// ENCADRES PAR : DARMONT Jrome                                              //
//                UNIVERSITE LYON 2 - LABORATOIRE ERIC                        //
//                http://eric.univ-lyon2.fr - - jerome.darmont@univ-lyon2.fr  //
//----------------------------------------------------------------------------//
// DESCRIPTION : Banc d'essai dcisionnel                                     //
//----------------------------------------------------------------------------//
// Cette application a pour but de permettre la cration d'un entrept de     //
// donnes synthtique et des charges associes.                              //
////////////////////////////////////////////////////////////////////////////////

package dweb03;

import java.sql.*;
import java.lang.*;

import javax.swing.JOptionPane;

////////////////////////////////////////////////////////////////////////////////
// CLASSE TableDimensionH                                                     //
//----------------------------------------------------------------------------//
// Cette classe sert  modliser les tables constituant les dimensions de     //
// l'entrept. Les requtes d'insertion et de selection sont xcutes au     //
// sein de cette classe et une "Connection SQL" est passe en paramtre des   //
// fonctions de cration, d'insertion et de selection dans la base            //
////////////////////////////////////////////////////////////////////////////////

public class TableDimensionH {

////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTS                                                                  //
////////////////////////////////////////////////////////////////////////////////
  private static String typeCles = "NUMBER";

  private String nomTable;
  private String nomClePrimaire;
  private String nomCleEtrangere = "";
  private TableDimensionH tableCleEtrangere;

  private int nbChampsDonnees;
  private String[] tNomChampsDonnees;
  private String[] tTypeChampsDonnees;

////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTEURS                                                              //
////////////////////////////////////////////////////////////////////////////////

  public TableDimensionH(String pNom, int pNbChpDonnees) {
    this.nomTable = pNom;
    this.nbChampsDonnees = pNbChpDonnees;
    this.tNomChampsDonnees = new String[this.nbChampsDonnees];
    this.tTypeChampsDonnees = new String[this.nbChampsDonnees];
  }

////////////////////////////////////////////////////////////////////////////////
// METHODES                                                                   //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNom - OK                                                       //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | nom de la table                                               //
////////////////////////////////////////////////////////////////////////////////
  public String getNom() {
    return this.nomTable;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getTypeCles - OK                                                  //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | type des cls primaires et trangres | ("INTEGER")           //
////////////////////////////////////////////////////////////////////////////////
  public String getTypeCles () {
    return this.typeCles;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getClePrimaire - OK                                               //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | nom du champ cl primaire de la table                         //
////////////////////////////////////////////////////////////////////////////////
  public String getClePrimaire () {
    return this.nomClePrimaire;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getCleEtrangere - OK                                              //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | nom du champ cl trangre de la table                        //
////////////////////////////////////////////////////////////////////////////////
  public String getCleEtrangere () {
    return this.nomCleEtrangere;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNbChamps - OK                                                   //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | nombre de champs de la table                                     //
////////////////////////////////////////////////////////////////////////////////
  public int getNbChamps() {
    return this.nbChampsDonnees;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setClePrimaire - OK                                               //
//----------------------------------------------------------------------------//
// E : pClePrimaire | String | nom du champ cl primaire                      //
// S : \                                                                      //
////////////////////////////////////////////////////////////////////////////////
  public void setClePrimaire (String pClePrimaire) {
    this.nomClePrimaire = pClePrimaire;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setCleEtrangere - OK                                              //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pTable | TableDimensionH | table rfrence par la cl trangre       //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la cration de la cl trangre s'est bien passe //
////////////////////////////////////////////////////////////////////////////////
  public boolean setCleEtrangere (Connection pConnexion, TableDimensionH pTable, String[] pMessage) {
    String requete = "";
    boolean resultat = true;

    this.tableCleEtrangere = pTable;
    this.nomCleEtrangere = "FK_" + this.tableCleEtrangere.getClePrimaire();

    requete = "ALTER TABLE " + this.nomTable + " ADD " + this.nomCleEtrangere + " " + this.typeCles;
    resultat &= this.executerUpdate(pConnexion,requete,pMessage);
    requete = "ALTER TABLE " + this.nomTable + " ADD FOREIGN KEY (" + this.nomCleEtrangere + ") REFERENCES " + this.tableCleEtrangere.getNom() + "(" + this.tableCleEtrangere.getClePrimaire() + ")";
    resultat &= this.executerUpdate(pConnexion,requete,pMessage);

    return resultat;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setNomChamp - OK                                                  //
//----------------------------------------------------------------------------//
// E : pNoChp | int | n du champ  nommer                                    //
//     pNomChamp | String | nom  donner au champ                             //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si le champ  bien t nomm                         //
////////////////////////////////////////////////////////////////////////////////
  public boolean setNomChamp (int pNoChp, String pNomChamp, String[] pMessage) {
    if (pNoChp > 0 && pNoChp <= this.nbChampsDonnees) {
      this.tNomChampsDonnees[pNoChp-1] = pNomChamp;
      return true;
    }
    else
      return false;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNomChamp - OK                                                  //
//----------------------------------------------------------------------------//
// E : pNoChp | int | n du champ                                             //
// S : String | nom du champ                                                  //
////////////////////////////////////////////////////////////////////////////////
  public String getNomChamp (int pNoChp) {
    if (pNoChp >= 0 && pNoChp < this.nbChampsDonnees)
      return this.tNomChampsDonnees[pNoChp];
    else
      return "";
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setNomsChampsDefaut - OK                                          //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : \                                                                      //
//----------------------------------------------------------------------------//
// un nom est donn  tous les champs de la table : "NOM_TABLE_DESCRi"        //
////////////////////////////////////////////////////////////////////////////////
  public void setNomsChampsDefaut () {
    int i;
    for (i=0; i<this.nbChampsDonnees; i++) {
      this.tNomChampsDonnees[i] = this.nomTable + "_DESCR" + Integer.toString(i+1);
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setTypeChamp - OK                                                 //
//----------------------------------------------------------------------------//
// E : pNoChp | int | n du champ dont on veut definir le type                //
//     pTypeChamp | String | type de donnes SQL du champ                     //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si le champ  bien t typ                         //
////////////////////////////////////////////////////////////////////////////////
  public boolean setTypeChamp (int pNoChp, String pTypeChamp, String[] pMessage) {
    if (pNoChp > 0 && pNoChp <= this.nbChampsDonnees) {
      this.tTypeChampsDonnees[pNoChp-1] = pTypeChamp;
      return true;
    }
    else
      return false;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getTypeChamp - OK                                                 //
//----------------------------------------------------------------------------//
// E : pNoChp | int | n du champ                                             //
// S : String | type du champ                                                 //
////////////////////////////////////////////////////////////////////////////////
  public String getTypeChamp (int pNoChp) {
    if (pNoChp >= 0 && pNoChp < this.nbChampsDonnees)
      return this.tTypeChampsDonnees[pNoChp];
    else
      return "";
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setTypesChampsDefaut - OK                                         //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : \                                                                      //
//----------------------------------------------------------------------------//
// dfinit un type par dfaut pour tous les champs de donnes : "VARCHAR2"    //
////////////////////////////////////////////////////////////////////////////////
  public void setTypesChampsDefaut () {
    int i;
    for (i=0; i < this.nbChampsDonnees; i++) {
      this.tTypeChampsDonnees[i] = "VARCHAR2(25)";
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION calculerTailleEnr - OK                                            //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : \                                                                      //
//----------------------------------------------------------------------------//
// retourne la taille d'un enregistrement en octets                           //
////////////////////////////////////////////////////////////////////////////////
  public int calculerTailleEnr () {
    int i;
    int tailleEnr = 0;

    // taille de la cl primaire
    tailleEnr += 16;
    // taille de la cl trangre si elle existe
    if (this.nomCleEtrangere != "")
      tailleEnr += 16;
    // taille des autres champs :  complter si d'autres types que VARCHAR2(25) sont utiliss
    for (i=0; i<this.nbChampsDonnees; i++) {
      if (this.tTypeChampsDonnees[i] == "VARCHAR2(25)")
        tailleEnr += 25;
    }
    return tailleEnr;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION creerTableBase - OK                                               //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la cration de la table s'est bien passe         //
//----------------------------------------------------------------------------//
// on cr la contrainte de cl primaire  la cration de la table            //
////////////////////////////////////////////////////////////////////////////////
  public boolean creerTableBase (Connection pConnexion, String[] pMessage) {
    int i;
    String rListeChamps = "";
    String requete = "";

    // construction des la liste des champs a crer dans la table
    rListeChamps += this.nomClePrimaire + " " + this.typeCles + " NOT NULL PRIMARY KEY";

    for (i=0; i < this.nbChampsDonnees ; i++) {
      rListeChamps += ", " + this.tNomChampsDonnees[i] + " " + this.tTypeChampsDonnees[i];
    }

    // Insertion de la table dans la base
    requete = "CREATE TABLE " + this.nomTable + " (" + rListeChamps + ")";
    return this.executerUpdate(pConnexion,requete,pMessage);
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION viderTableBase - OK                                               //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si les enregistrements ont bien t supprims        //
////////////////////////////////////////////////////////////////////////////////
  public boolean viderTableBase (Connection pConnexion, String[] pMessage) {
    String requete = "DELETE FROM " + this.nomTable;
    return this.executerUpdate(pConnexion,requete,pMessage);
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION effacerTableBase - OK                                             //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la suppression de la table s'est bien passe      //
////////////////////////////////////////////////////////////////////////////////
  public boolean effacerTableBase (Connection pConnexion, String[] pMessage) {
    String requete = "DROP TABLE " + this.nomTable;
    return this.executerUpdate(pConnexion,requete,pMessage);
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getEnregistrementAleatoire - OK                                   //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : int | cl primaire de l'enregistrement slectionn                     //
//----------------------------------------------------------------------------//
// on utilise la fonction "ORDER BY dbms_random.random" de SQL                //
////////////////////////////////////////////////////////////////////////////////
  public int getEnregistrementAleatoire (Connection pConnexion, String[] pMessage) {
    int clePrimaire;
    ResultSet retourRequete = null;
    String requete = "SELECT * FROM " + this.getNom() + " order by DBMS_RANDOM.random";

    try {
      Statement statement = pConnexion.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
      retourRequete = statement.executeQuery(requete);

      if (retourRequete.first())
        clePrimaire = retourRequete.getInt(1);
      else
        clePrimaire = 0;

      // bien fermer le statement pour libration de la mmoire !
      retourRequete.close();
      statement.close();
      return clePrimaire;
    }
    catch (SQLException e) {
      System.out.println("--> SQLException : " + e);
      System.out.println("--> " + requete);
      pMessage[0] += "--> SQLException : " + e + "\n";
      return 0;
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getEnregistrements - OK                                           //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : ResultSet | contient le tableau des cl primaires                      //
//----------------------------------------------------------------------------//
// on utilise la fonction "ORDER BY dbms_random.random" de SQL                //
////////////////////////////////////////////////////////////////////////////////
  public ResultSet getEnregistrements (Connection pConnexion, String[] pMessage) {
    String requete = "SELECT " + this.getClePrimaire() + " FROM " + this.getNom();

    try {
		ResultSet rs;
      Statement statement = pConnexion.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);

	  rs = statement.executeQuery(requete);

	  //statement.Close();
      return rs;
    }
    catch (SQLException e) {
      System.out.println("--> SQLException : " + e);
      System.out.println("--> " + requete);
      pMessage[0] += "--> SQLException : " + e + "\n";
      return null;
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getClePrimaireLibre - OK                                          //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : int | cl primaire libre dans la table                                 //
//----------------------------------------------------------------------------//
// pour minimiser le nombre d'oprations on cherche la valeur max des cls    //
// primaires et on y ajoute 1 pour obtenir une cl libre                      //
// si pas d'enregistrements dans la table on retourne 1                       //
////////////////////////////////////////////////////////////////////////////////
  public int getClePrimaireLibre (Connection pConnexion, String[] pMessage) {
    int clePrimaireLibre = 0;
    ResultSet retourRequete = null;
    String requete = "SELECT MAX(" + this.nomClePrimaire + ") FROM " + this.nomTable;

    try {
      Statement statement = pConnexion.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
      retourRequete = statement.executeQuery(requete);

      if (retourRequete.first())
        clePrimaireLibre = retourRequete.getInt(1) + 1;
      else
        clePrimaireLibre = 1;

      retourRequete.close();
      statement.close();
      return clePrimaireLibre;
    }
    catch (SQLException e) {
      System.out.println("--> SQLException : " + e);
      System.out.println("--> " + requete);
      pMessage[0] += "--> SQLException : " + e + "\n";
      return 0;
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION ajouterEnregistrement - OK                                        //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si l'enregistrement est bien insr                  //
////////////////////////////////////////////////////////////////////////////////
  public boolean ajouterEnregistrement (Connection pConnexion, String[] pMessage) {
    int i;
    int clePrimaire, cleEtrangere;
    String requete = "";
    String listeChamps = "";
    String listeValeurs = "";

    //JOptionPane.showMessageDialog(null,"--> entree ajouter enre insert dimension ");

    // On rcupre une cl primaire libre
    clePrimaire = getClePrimaireLibre (pConnexion, pMessage);

    if (clePrimaire != 0) {
      listeChamps += this.nomClePrimaire;
      listeValeurs += Integer.toString(clePrimaire);

      // On remplit les champs avec des chaines de caractres alatoires
      for (i=0; i < this.nbChampsDonnees; i++) {
        listeChamps += ", " + this.tNomChampsDonnees[i];
        if (this.tTypeChampsDonnees[i] == "VARCHAR2(25)") {
          listeValeurs += ", DBMS_RANDOM.string('a',25)";
        }
      }

      // Si il y a une cl trangre dans la table on recupre une valeur dans la table rfrence
      if (this.nomCleEtrangere != "") {
        cleEtrangere = this.tableCleEtrangere.getEnregistrementAleatoire(pConnexion, pMessage);
        listeChamps += ", " + this.nomCleEtrangere;
        listeValeurs += ", " + Integer.toString(cleEtrangere);
      }

      requete = "INSERT INTO " + this.nomTable + "(" + listeChamps + ") VALUES (" + listeValeurs + ")";
      //JOptionPane.showMessageDialog(null,"--> insert dimension "+requete);
      return executerUpdate(pConnexion,requete,pMessage);
    }
    else
      return false;
  }


////////////////////////////////////////////////////////////////////////////////
//FONCTION modifierEnregistrement - OK                                        //
//----------------------------------------------------------------------------//
//E : pConnexion | Connection | connexion a utiliser                         //
//    pCle | int | cl primaire												 //
//    pMessage | String  | tableau de chaines pour rcupration des messages //
//S : boolean | indique si l'enregistrement est bien insr                  //
////////////////////////////////////////////////////////////////////////////////
 public boolean modifierEnregistrement (Connection pConnexion, int pCle, String[] pMessage) {
   int i;
   int clePrimaire, cleEtrangere;
   String requete = "";
   String listeChamps = "";

   // On rcupre une cl primaire libre
   clePrimaire = pCle;

   //JOptionPane.showMessageDialog(null, "modifier enregistrement");

   if (clePrimaire != 0) {

	 listeChamps += this.nomClePrimaire + " = ";
	 listeChamps += Integer.toString(clePrimaire);

	 // On remplit les champs avec des chaines de caractres alatoires
     for (i=0; i < this.nbChampsDonnees; i++) {
       listeChamps += ", " + this.tNomChampsDonnees[i] + " = ";
       if (this.tTypeChampsDonnees[i] == "VARCHAR2(25)") {
         listeChamps += "DBMS_RANDOM.string('a',25)";
       }
     }

     // Si il y a une cl trangre dans la table on recupre une valeur dans la table rfrence
     if (this.nomCleEtrangere != "") {
       cleEtrangere = this.tableCleEtrangere.getEnregistrementAleatoire(pConnexion, pMessage);
       listeChamps += ", "+ this.nomCleEtrangere + " = ";
       listeChamps+= Integer.toString(cleEtrangere);
     }
     requete = "UPDATE " + this.nomTable + " SET " + listeChamps + " WHERE " + this.nomClePrimaire + " = " + String.valueOf(clePrimaire);

	try {
      Statement statement = pConnexion.createStatement();
      statement.executeUpdate(requete);
      statement.close();
      return true;
    }
     catch (SQLException e) {
      System.out.println("--> SQLException modifier enregistrement Dim: " + e);
      System.out.println("--> " + requete);
      pMessage[0] += "--> SQLException : modifier enregistrement Dim " + e + "\n";
      pMessage[0] += "--> " + requete;
      return false;
    }
   }
   else
     return false;
 }

////////////////////////////////////////////////////////////////////////////////
// FONCTION executerUpdate - OK                                               //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pRequete | String | requete d'update  excuter sur la table           //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la requte est bien xcute                      //
//----------------------------------------------------------------------------//
// Fonction uniquement dstine  ne pas repter le code d'xcution de       //
// la requte.                                                                //
// Fonction private uniquement appele par les fonctions de la classe         //
////////////////////////////////////////////////////////////////////////////////
  private boolean executerUpdate (Connection pConnexion, String pRequete, String[] pMessage) {
    try {
      Statement statement = pConnexion.createStatement();
      statement.executeUpdate(pRequete);
      statement.close();
      return true;
    }
    catch (SQLException e) {
      System.out.println("--> SQLException : " + e);
      System.out.println("--> " + pRequete);
      pMessage[0] += "--> SQLException : " + e + "\n";
      pMessage[0] += "--> " + pRequete;
      return false;
    }
  }
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//FONCTION nbreEnregistrement - OK                                      //
//----------------------------------------------------------------------------//
//E : \                                                                      //
//S : \                                                                      //
//----------------------------------------------------------------------------//
////
////////////////////////////////////////////////////////////////////////////////
public long nbreEnregistrementDim (Connection pConnexion){
   int nbEnr = 0;
   ResultSet rs;
   String pRequete;

   pRequete = "SELECT COUNT(*) FROM "+this.getNom();

   try{
	   Statement statement = pConnexion.createStatement();

	   rs = statement.executeQuery(pRequete);
	   rs.next();
	   nbEnr += rs.getInt(1);
	   rs.close();
	   statement.close();
   }
   catch (SQLException e) {
   }
   return nbEnr;
}
}
