////////////////////////////////////////////////////////////////////////////////
// TITRE : DWEB - DATAWAREHOUSE ENGINEERING BENCHMARK - VERSION 0.4           //
//----------------------------------------------------------------------------//
// AUTEURS : JOUBERT Bruno / GUESMIA Sofiane                                  //
//           UNIVERSITE LYON 2 - MASTER IIDEE                                 //
//----------------------------------------------------------------------------//
// English translation by J. Darmont                                          //
// - Step 1: Interface (DONE)                                                 //
// - Step 2: Code/comments (TO DO)                                            //
//----------------------------------------------------------------------------//
// ENCADRES PAR : DARMONT Jrome                                              //
//                UNIVERSITE LYON 2 - LABORATOIRE ERIC                        //
//                http://eric.univ-lyon2.fr - - jerome.darmont@univ-lyon2.fr  //
//----------------------------------------------------------------------------//
// DESCRIPTION : Banc d'essai dcisionnel                                     //
//----------------------------------------------------------------------------//
// Cette application a pour but de permettre la cration d'un entrept de     //
// donnes synthtique et des charges associes.                              //
////////////////////////////////////////////////////////////////////////////////

package dweb03;

import java.sql.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;

////////////////////////////////////////////////////////////////////////////////
// CLASSE TableFait                                                           //
//----------------------------------------------------------------------------//
// Cette classe sert  modliser les tables de faits de l'entrept.           //
////////////////////////////////////////////////////////////////////////////////

public class TableFait {

////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTS                                                                  //
////////////////////////////////////////////////////////////////////////////////
  private static String typeMesures = "NUMBER";

  private String nomTableFait;
  private double densiteTF;

  private int nbDimensionsTF;
  private DimensionH[] tDimensionsTF;

  private int nbMesuresTF;
  private String[] tMesuresTF;

////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTEURS                                                              //
////////////////////////////////////////////////////////////////////////////////

  public TableFait(String pNom, double pDensite, int pNbMesures, int pNbDimensions) {
    this.nomTableFait = pNom;
    this.densiteTF = pDensite;
    this.nbMesuresTF = pNbMesures;
    this.nbDimensionsTF = pNbDimensions;
    this.tMesuresTF = new String[this.nbMesuresTF];
    this.tDimensionsTF = new DimensionH[this.nbDimensionsTF];
  }

////////////////////////////////////////////////////////////////////////////////
// METHODES                                                                   //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNom - OK                                                       //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | nom de la table de faits                                      //
////////////////////////////////////////////////////////////////////////////////
  public String getNom () {
    return this.nomTableFait;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getInfos - OK                                                     //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : String | infos de la table de faits en vue de l'affichage dans une     //
//     boite de dialogue                                                      //
////////////////////////////////////////////////////////////////////////////////
  public String getInfos () {
    int i;
    String infos = "";

    infos += this.nomTableFait + " : Associated dimensions = {";

    for (i=0; i<this.nbDimensionsTF; i++) {
      infos += " " + this.getDimensionI(i).getNom();
    }

    infos += " }";

    infos += " / Nb of measures = " + Integer.toString(this.nbMesuresTF)
           + " / Density = " + Double.toString(this.densiteTF);

    return infos;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getDensite - OK                                                   //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : double | densit de la table de faits                                  //
////////////////////////////////////////////////////////////////////////////////
  public double getDensite () {
    return this.densiteTF;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNbMesures - OK                                                 //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | nombre de champs de la table                                     //
////////////////////////////////////////////////////////////////////////////////
  public int getNbMesures() {
    return this.nbMesuresTF;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNbDimensions - OK                                              //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : int | nb de dimensions associes  la table de faits                   //
////////////////////////////////////////////////////////////////////////////////
  public int getNbDimensions () {
    return this.nbDimensionsTF;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setNbDimensions - OK                                              //
//----------------------------------------------------------------------------//
// E : pNbDim | int | nb de dimensions  associer  la table de faits         //
// S : \                                                                      //
////////////////////////////////////////////////////////////////////////////////
  public void setNbDimensions (int pNbDim) {
    this.nbDimensionsTF = pNbDim;
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION setNomsChampsMesuresDefaut - OK                                   //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : \                                                                      //
//----------------------------------------------------------------------------//
// Donne un nom par dfaut aux champs de mesures                              //
////////////////////////////////////////////////////////////////////////////////
  public void setNomsChampsMesuresDefaut () {
    int i;
    for (i=0; i<this.nbMesuresTF; i++) {
      this.tMesuresTF[i] = this.nomTableFait + "_MEASURE" + Integer.toString(i+1);
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getNomChampMesure - OK                                            //
//----------------------------------------------------------------------------//
// E : pNoChp | int | n du champ                                             //
// S : String | nom du champ                                                  //
////////////////////////////////////////////////////////////////////////////////
  public String getNomChampMesure (int pNoChp) {
    if (pNoChp >= 0 && pNoChp < this.nbMesuresTF)
      return this.tMesuresTF[pNoChp];
    else
      return "";
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION calculerTaille - OK                                               //
//----------------------------------------------------------------------------//
// E : \                                                                      //
// S : \                                                                      //
//----------------------------------------------------------------------------//
// retourne la taille de la table en octets                                   //
////////////////////////////////////////////////////////////////////////////////
  public long calculerTaille () {
    int i;
    int tailleEnr = 0;
    int nbEnr = 1;
    long tailleTable = 0;

    for (i=0; i<this.nbDimensionsTF; i++) {
      if (this.getDimensionI(i).getTableI(0).getTypeCles() == "NUMBER")
        tailleEnr += 16;

      nbEnr *= this.getDimensionI(i).getNbEnregistrementsHautNiveau() *
               Math.pow(this.getDimensionI(i).getFacteurEchelle(),
               this.getDimensionI(i).getProfondeur() - 1);
    }
    nbEnr *= this.densiteTF;

    for (i=0; i<this.nbMesuresTF; i++) {
      if (this.typeMesures == "NUMBER")
        tailleEnr += 16;
    }
    tailleTable = (long)nbEnr * (long)tailleEnr;
    return tailleTable;
  }

////////////////////////////////////////////////////////////////////////////////
//FONCTION nbreEnregistrementFait - OK                                            //
//----------------------------------------------------------------------------//
//E : \                                                                      //
//S : \                                                                      //
//----------------------------------------------------------------------------//
//retourne le nombre d'enregistrements de la table                            //
////////////////////////////////////////////////////////////////////////////////
 public long nbreEnregistrementFait (Connection pConnexion) {
   long nbEnr = 0;
   ResultSet rs;
   String pRequete = "SELECT COUNT(*) FROM "+this.nomTableFait;

   try{
	   Statement statement = pConnexion.createStatement();

	   rs = statement.executeQuery(pRequete);
	   rs.next();
	   nbEnr = rs.getInt(1);
	   rs.close();
	   statement.close();
   }
   catch (SQLException e) {
   }
   return nbEnr;
 }

////////////////////////////////////////////////////////////////////////////////
// FONCTION ajouterDimension - OK                                             //
//----------------------------------------------------------------------------//
// E : pRang | int | rang d'insertion de la dimension dans le tableau         //
//     pDimension | DimensionH | dimension  relier  la table de fait        //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la dimension a bien t ajoute                   //
//      la table de faits                                                    //
////////////////////////////////////////////////////////////////////////////////
  public boolean ajouterDimension (int pRang, DimensionH pDimension, String[] pMessage) {
    if (pRang < 0 || pRang > this.nbDimensionsTF) {
      pMessage[0] += "Erreur lors de l'ajout de la dimension " + pDimension.getNom() + " dans la table de fait " + this.getNom() + "\n";
      return false;
    }
    else {
      this.tDimensionsTF[pRang] = pDimension;
      return true;
    }
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION getDimensionI - OK                                                //
//----------------------------------------------------------------------------//
// E : pRang | int | rang de la dimension que l'on veut rcuprer             //
// S : DimensionH | dimension au rang i si valide, null sinon                 //
////////////////////////////////////////////////////////////////////////////////
  public DimensionH getDimensionI (int pRang) {
    if (pRang < 0 || pRang > this.nbDimensionsTF)
      return null;
    else
      return this.tDimensionsTF[pRang];
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION genererTableFait - OK                                             //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la dimension a bien t gnre                   //                                                                      //
////////////////////////////////////////////////////////////////////////////////
  public boolean genererTableFait(Connection pConnexion, String[] pMessage) {
    int i;
    String rListeChamps = "";
    String rPrimary = "";
    String requete = "";

    rPrimary += "PRIMARY KEY (";

    for (i=0; i < this.nbDimensionsTF; i++) {
      rListeChamps += "FK_" + this.tDimensionsTF[i].getTableI(0).getClePrimaire() + " "
                      + this.tDimensionsTF[i].getTableI(0).getTypeCles()
                      + " NOT NULL";
      rPrimary += "FK_" + this.tDimensionsTF[i].getTableI(0).getClePrimaire();
      if (i < this.getNbDimensions()-1) {
        rListeChamps += ", ";
        rPrimary += ", ";
      }
    }

    rPrimary += ")";

    for (i=0; i < this.nbMesuresTF ; i++) {
      rListeChamps += ", " + this.tMesuresTF[i] + " " + this.typeMesures;
    }

    // Insertion de la table dans la base
    requete = "CREATE TABLE " + this.nomTableFait + " (" + rListeChamps + ", " + rPrimary + ")";
    return this.executerUpdate(pConnexion,requete,pMessage);
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION viderTableBase - OK                                               //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si les enregistrements ont bien t supprims        //
////////////////////////////////////////////////////////////////////////////////
  public boolean viderTableBase (Connection pConnexion, String[] pMessage) {
    String requete = "DELETE FROM " + this.nomTableFait;
    return this.executerUpdate(pConnexion,requete,pMessage);
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION effacerTableBase - OK                                             //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la suppression de la table s'est bien passe      //
////////////////////////////////////////////////////////////////////////////////
  public boolean effacerTableBase (Connection pConnexion, String[] pMessage) {
    String requete = "DROP TABLE " + this.nomTableFait;
    return this.executerUpdate(pConnexion,requete,pMessage);
  }

////////////////////////////////////////////////////////////////////////////////
// FONCTION ajouterEnregistrement - OK                                        //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si l'enregistrement est bien insr                  //
////////////////////////////////////////////////////////////////////////////////
  public boolean ajouterEnregistrement (Connection pConnexion, String pClesPrimaires, String pValeursClesPrimaires, String[] pMessage) {
    int i;
    String listeChamps = pClesPrimaires;
    String listeValeurs = pValeursClesPrimaires;
    String requete;
    Random generateur = new Random( );

    for (i=0; i<this.nbMesuresTF; i++) {
      listeChamps += ", " + this.tMesuresTF[i];
      listeValeurs += ", " + Integer.toString(generateur.nextInt());
    }
    requete = "INSERT INTO " + this.nomTableFait + "(" + listeChamps + ") VALUES (" + listeValeurs + ")";
    return executerUpdate (pConnexion, requete, pMessage);
  }

////////////////////////////////////////////////////////////////////////////////
//FONCTION modifierEnregistrement - OK                                        //
//----------------------------------------------------------------------------//
//E : pConnexion | Connection | connexion a utiliser                         //
//    pCleP | String | champs SET du UPDATE 								 //
//    pMessage | String  | tableau de chaines pour rcupration des messages //
//S : boolean | indique si l'enregistrement est bien insr                  //
////////////////////////////////////////////////////////////////////////////////
 public boolean modifierEnregistrement (Connection pConnexion, String pCleP, String[] pMessage) {
   int i;
   String listeChamps = pCleP;
   String listeMesures = "";
   String requete;
   Random generateur = new Random( );

	   for (i=0; i<this.nbMesuresTF; i++) {
	     listeMesures += this.tMesuresTF[i] + " = ";
	     listeMesures += Integer.toString(generateur.nextInt());

	     if (i != this.nbMesuresTF-1) {
	         listeMesures += ", ";
	       }
	   }
	   requete = "UPDATE " + this.nomTableFait + " SET " + listeMesures + " WHERE " + listeChamps;

   try{
	      Statement statement = pConnexion.createStatement();
	      statement.executeUpdate(requete);
	      statement.clearBatch();
	      statement.close();
	      return true;
   }
   catch (SQLException e) {
	      System.out.println("--> SQLException modifier enregistrement : " + e);
	      System.out.println("--> " + requete);
	      pMessage[0] += "--> SQLException : modifier enregistrement " + e + "\n";
	      pMessage[0] += "--> " + requete;
	      return true;
}

 }

////////////////////////////////////////////////////////////////////////////////
// FONCTION remplirTable - OK                                                 //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la table est bien remplie                         //
////////////////////////////////////////////////////////////////////////////////
  public boolean remplirTable (Connection pConnexion, JProgressBar pProgressBar, String[] pMessage) {
    int i,j;
    int nbEnr = 1;
    boolean passageEnrSuiv;
    boolean resultat = true;

    String lClesPrimaires = "";
    String lValeursClesPrimaires = "";

    Random generateur = new Random( );

    ResultSet[] tablesDims = new ResultSet[this.getNbDimensions()];

    try {
      // On calcule le produit cardinal des dimensions
      for (i=0; i < this.getNbDimensions(); i++) {
        nbEnr *= this.getDimensionI(i).getNbEnregistrementsHautNiveau() *
                 Math.pow(this.getDimensionI(i).getFacteurEchelle(),
                 this.getDimensionI(i).getProfondeur()-1);

        // On recupere le contenu des tables de bas niveau des dimensions : gain de temps pour la suite
        tablesDims[i] = this.getDimensionI(i).getTableI(0).getEnregistrements(pConnexion, pMessage);

        // On se place sur le premier enregistrement de chaque table
        if (!tablesDims[i].first())
          return false;
      }

      // Initialisation de la barre de progression
      pProgressBar.setMinimum(0);
      pProgressBar.setMaximum(nbEnr);

      // boucle sur le produit cardianl des dimensions
      // on est sur de cette facon de ne pas avoir deux fois le mme enregistrement
      for (i=0; i<nbEnr; i++) {
        // MAJ de la barre de progression
        pProgressBar.setValue(i);

        // generateur.nextDouble() compris entre 0.0 et 1.0 et repartition normale
        if (generateur.nextDouble() <= this.densiteTF){
          lClesPrimaires = "";
          lValeursClesPrimaires = "";

          // on construit les chaines des noms des cles primaires et des valeurs associes
          for (j=0; j<this.getNbDimensions(); j++) {
            lClesPrimaires += "FK_" + this.getDimensionI(j).getTableI(0).getClePrimaire();
            lValeursClesPrimaires += Integer.toString(tablesDims[j].getInt(1));

            if (j != this.getNbDimensions()-1) {
              lClesPrimaires += ", ";
              lValeursClesPrimaires += ", ";
            }
          }
          // on ajoute l'enregistrement dans la table
          resultat &= this.ajouterEnregistrement(pConnexion,lClesPrimaires,lValeursClesPrimaires,pMessage);
        }

        // positionnement sur l'enregistrement suivant dans la liste
        passageEnrSuiv = false;
        j = this.getNbDimensions()-1;

        while (!passageEnrSuiv && j >= 0) {
          if (tablesDims[j].next()) {
            passageEnrSuiv = true;
          }
          else {
            tablesDims[j].first();
            j--;
          }
        }
      }
      // fermeture des resultsets temporaires
      for (i=0; i < this.getNbDimensions(); i++) {
        tablesDims[i].close();
      }
      return resultat;
    }
    catch (Exception e) {
      System.out.println("--> Exception : " + e);
      pMessage[0] += "--> Exception : " + e + "\n";
      return false;
    }
  }

////////////////////////////////////////////////////////////////////////////////
//FONCTION ajouterFaits - OK                                                 //
//----------------------------------------------------------------------------//
//E : pConnexion | Connection | connexion a utiliser    					 //
//    pMessage | String  | tableau de chaines pour rcupration des messages //
//S : boolean | indique si la table est bien remplie                         //
////////////////////////////////////////////////////////////////////////////////
 public boolean ajouterFaits (Connection pConnexion, String tFait, String[] pMessage) {
   int i;
   boolean resultat = true;
   String pk ="", dim = "", fk = "", requete = "", valeurCle = "";
   ResultSet rs;

   for(i=0; i<this.nbDimensionsTF; i++){
	   pk += "PK_" + this.getDimensionI(i).getNom() + "_TAB1";
	   dim += this.getDimensionI(i).getNom() + "_TAB1";
	   fk += "FK_PK_" + this.getDimensionI(i).getNom() + "_TAB1";

	   if(i != (this.nbDimensionsTF-1)){
		   pk += ", ";
		   dim += ", ";
		   fk += ", ";
	   }
   }

   requete += "SELECT "+pk+ " FROM " + dim + " MINUS SELECT "+fk+ " FROM "+tFait;

   try{
	   Statement statement = pConnexion.createStatement();

	   rs = statement.executeQuery(requete);
	   rs.next();
	   for(i=0; i<this.nbDimensionsTF; i++){
		   valeurCle += Integer.toString(rs.getInt(i+1));

		   if(i != (this.nbDimensionsTF-1)){
			   valeurCle += ", ";
		   }
	   }

	   // bien fermer le statement pour libration de la mmoire !
	   rs.close();
	   statement.close();
   }
   catch (SQLException e) {
	   	System.out.println("--> Exception : " + e);
	    pMessage[0] += "--> Exception : " + e + "\n";
	   	return false;
   }

   resultat &= this.ajouterEnregistrement(pConnexion,fk,valeurCle,pMessage);

   return resultat;

 }


////////////////////////////////////////////////////////////////////////////////
//FONCTION getClePrimaireAleatoire - OK                                                 //
//----------------------------------------------------------------------------//
//E : pConnexion | Connection | connexion a utiliser    					 //
//   pMessage | String  | tableau de chaines pour rcupration des messages //
//S : String |                          //
////////////////////////////////////////////////////////////////////////////////
public String getClePrimaireAleatoire (Connection pConnexion, String[] pMessage) {
  int i,j;

  String lClesPrimaires = "";

  //ResultSet[] tablesDims = new ResultSet[this.getNbDimensions()];
  ResultSet rs;

  try {

    // on construit les chaines des noms des cles primaires et des valeurs associes
    for (j=0; j<this.getNbDimensions(); j++) {
      lClesPrimaires += "FK_" + this.getDimensionI(j).getTableI(0).getClePrimaire()+" = ";

      rs = this.getDimensionI(j).getTableI(0).getEnregistrements(pConnexion, pMessage);
      rs.next();

      lClesPrimaires += Integer.toString(/*tablesDims[j].getInt(1)*/rs.getInt(1));

      if (j != this.getNbDimensions()-1) {
        lClesPrimaires += " AND ";
      }
    }

    //rs.close();
    return lClesPrimaires;
  }
  catch (Exception e) {
    System.out.println("--> Exception : " + e);
    pMessage[0] += "--> Exception : " + e + "\n";
    return "";
  }
}


////////////////////////////////////////////////////////////////////////////////
// FONCTION executerUpdate - OK                                               //
//----------------------------------------------------------------------------//
// E : pConnexion | Connection | connexion a utiliser                         //
//     pRequete | String | requete d'update  excuter sur la table           //
//     pMessage | String  | tableau de chaines pour rcupration des messages //
// S : boolean | indique si la requte est bien xcute                      //
//----------------------------------------------------------------------------//
// Fonction uniquement dstine  ne pas repter le code d'xcution de       //
// la requte.                                                                //
// Fonction private uniquement appele par les fonctions de la classe         //
////////////////////////////////////////////////////////////////////////////////
  private boolean executerUpdate (Connection pConnexion, String pRequete, String[] pMessage) {
    try {
      Statement statement = pConnexion.createStatement();
      statement.executeUpdate(pRequete);
      statement.close();
      return true;
    }
    catch (SQLException e) {
      System.out.println("--> SQLException : " + e);
      System.out.println("--> " + pRequete);
      pMessage[0] += "--> SQLException : " + e + "\n";
      pMessage[0] += "--> " + pRequete;
      return false;
    }
  }
////////////////////////////////////////////////////////////////////////////////
}
